@@ -2059,6 +2059,44 @@ trace_setdents (call_frame_t *frame,
   return 0;
 }
 
+
+int32_t
+trace_checksum_cbk (call_frame_t *frame,
+		    void *cookie,
+		    xlator_t *this,
+		    int32_t op_ret,
+		    int32_t op_errno,
+		    uint8_t *fchecksum,
+		    uint8_t *dchecksum)
+{
+  gf_log (this->name, GF_LOG_NORMAL, 
+	  "op_ret (%d), op_errno(%d)", op_ret, op_errno);
+
+  STACK_UNWIND (frame, op_ret, op_errno, fchecksum, dchecksum);
+
+  return 0;
+}
+
+int32_t
+trace_checksum (call_frame_t *frame,
+		xlator_t *this,
+		loc_t *loc,
+		int32_t flag)
+{
+  gf_log (this->name, GF_LOG_NORMAL, 
+	  "loc->path (%s) flag (%d)", loc->path, flag);
+  
+  STACK_WIND (frame,
+	      trace_checksum_cbk,
+	      FIRST_CHILD(this), 
+	      FIRST_CHILD(this)->fops->checksum, 
+	      loc,
+	      flag);
+
+  return 0;
+}
+
+
 #ifndef GF_SOLARIS_HOST_OS
 void
 enable_all_calls (int enabled)
@@ -2221,6 +2259,7 @@ struct xlator_fops fops = {
   .forget      = trace_forget,
   .setdents    = trace_setdents,
   .getdents    = trace_getdents,
+  .checksum    = trace_checksum,
 };
 
 int32_t 
@@ -2254,43 +2293,6 @@ trace_stats (call_frame_t *frame,
   return 0;
 }
 
-int32_t
-trace_checksum_cbk (call_frame_t *frame,
-		    void *cookie,
-		    xlator_t *this,
-		    int32_t op_ret,
-		    int32_t op_errno,
-		    uint8_t *fchecksum,
-		    uint8_t *dchecksum)
-{
-  gf_log (this->name, GF_LOG_NORMAL, 
-	  "op_ret (%d), op_errno(%d)", op_ret, op_errno);
-
-  STACK_UNWIND (frame, op_ret, op_errno, fchecksum, dchecksum);
-
-  return 0;
-}
-
-int32_t
-trace_checksum (call_frame_t *frame,
-		xlator_t *this,
-		loc_t *loc,
-		int32_t flag)
-{
-  gf_log (this->name, GF_LOG_NORMAL, 
-	  "loc->path (%s) flag (%d)", loc->path, flag);
-  
-  STACK_WIND (frame,
-	      trace_checksum_cbk,
-	      FIRST_CHILD(this), 
-	      FIRST_CHILD(this)->mops->checksum, 
-	      loc,
-	      flag);
-
-  return 0;
-}
-
 struct xlator_mops mops = {
   .stats    = trace_stats,
-  .checksum = trace_checksum,
 };
